/*
   ┏━━━┓╱╱╱╱╱┏┓╱╱┏┓╱╱╱╱╱┏┓╱╱   
   ┃┏━┓┃╱╱╱╱╱┃┃╱╱┃┃╱╱╱╱╱┃┃╱╱╱  
   ┃┗━┛┣┓┏┳━━┫┃┏┓┃┃╱╱┏━━┫┗━┳━━┓
   ┃┏━━┫┃┃┃┏┓┃┗┛┃┃┃╱┏┫┏┓┃┏┓┃━━┫
   ┃┃╱╱┃┗┛┃┃┃┃┏┓┃┃┗━┛┃┏┓┃┗┛┣━━┃
   ┗┛╱╱┗━━┻┛┗┻┛┗┛┗━━━┻┛┗┻━━┻━━┛
    ━━━━━━━━━━━━━━━━━━━━━━━━━━ 

	Copyright (c) 2023 Punk Labs LLC

	This file is part of OneTrick URCHIN

	OneTrick URCHIN is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation, either version 3 of the License, or (at your option)
	any later version.

	OneTrick URCHIN is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	OneTrick URCHIN.  If not, see <http://www.gnu.org/licenses/>.
*/

declare name "OneTrick URCHIN DSP";
declare copyright "Copyright (c) 2023 Punk Labs LLC";
declare license "GPLv3 (or later)";

import("stdfaust.lib");
import("shared.lib");

modWheel = nentry("ModWheel", 0, 0, 1, 0.001) : ot.smoothParam;

globalReverbSize = it.interpolate_linear(hslider("[010]Global_ReverbSize[export:Reverb Size][unit:%]", 15, 0, 100, 0.01) * 0.01, 0.1, 2); //(0.1-5) is a little much
globalReverbDiffusion = hslider("[020]Global_ReverbDiffusion[export:Reverb Diffusion][unit:%]", 50, 0, 100, 0.01) * 0.01;
globalReverbDamp = hslider("[030]Global_ReverbDamp[export:Reverb Damp][unit:%]", 25, 0, 100, 0.01) * 0.01;
globalReverbAmount = hslider("[040]Global_Reverb[export:Reverb][unit:%]", 50, 0, 100, 0.01) * 0.01;
//globalReverbFeedback = hslider("[050]Global_ReverbFeedback[export:Reverb Feedback][unit:%]", 15, 0, 100, 0.01) * 0.01;
globalReverbGate = hslider("[060]Globals_ReverbGate[export:Reverb Gate][unit:db]", -60, -60, 0, 0.01);
globalReverbCutoff = hslider("[060]Globals_ReverbHighpass[export:Reverb Highpass][unit:Hz]", 300, 100, 1000, 0.01);

globalVolume = ba.db2linear(max(-100, hslider("[070]Global_Volume[export:Volume][unit:db]", 0, -100, 6, 0.1)));
globalLowpassParam = hslider("[080]Global_Lowpass[export:Lowpass][unit:kHz]", 20, 2, 20, 0.01) * 1000 : min(MAX_FREQ);
globalHighpassParam = hslider("[090]Global_Highpass[export:Highpass][unit:Hz]", 20, 20, 200, 0.01);
globalLowpass =  interpolate_freq(modWheel, globalLowpassParam, 2000);
globalHighpass =  interpolate_freq(modWheel, globalHighpassParam, 200);

globalNoiseAmount = hslider("[100]Global_Noise[export:Noise Amount][unit:%]", 0, 0, 100, 0.01)/100;
globalNoiseType = hslider("[110]Global_NoiseType[export:Noise Type][enum:Vinyl,Tape]", 0, 0, 1, 1);
globalSaturation = hslider("[120]Global_Saturation[export:Saturation][unit:%]", 0, 0, 100, 0.01) * 0.01;
//globalSaturationAsym = hslider("[130]Global_Saturation_Asym[export:Saturation Asym][unit:%]", 0, 0, 100, 0.01) * 0.01 / 2;

globalSamplerate = hslider("[140]Global_SampleRate[export:Samplerate][unit:kHz]", 44.1, 11.025, 44.1, 0.01) * 1000 : min(ma.SR);
globalBitdepth = hslider("[150]Global_Bits[export:Bits][unit:bit]", 16, 4, 16, 1);


stereoTapeNoise = ((ot.noises(2, 0):filter), (ot.noises(2, 1):filter)) : stereo(*(ba.db2linear(-24))) with {
    filter = fi.lowshelf(1, 17, 100) : fi.lowpass(24, 16500 : min(MAX_FREQ));
};
stereoVinylNoise = ((noiseChannel(0)), (noiseChannel(1))) with {
    filter = fi.lowshelf(1, 17, 100) : ef.dryWetMixer(0.75,fi.lowpass(1, 500)) : fi.peak_eq(warble*10, 1500, 3500) : fi.lowpass(24, 8000);
    rpmFreq = 78/60;
    wobble1 = os.osc(rpmFreq) : it.remap(-1, 1, 0, 1);
    wobble2 = os.osc(rpmFreq*2) : it.remap(-1, 1, 0, 1);
    wobble4 = os.osc(rpmFreq*4) : it.remap(-1, 1, 0, 1);
    warble01 = it.interpolate_linear(wobble1*wobble2*wobble4, 0, 1);
    warble = it.interpolate_linear(warble01, 0.8, 1);
    noiseChannel(x) = ot.noises(4, x) + pop + crackle : *(warble) : *(ba.db2linear(-21)) : filter with {
        attackDecay = ot.noise : it.remap(-1, 1, 1, 8) : ba.downSample(10) : /(1000);
        popFilter = fi.bandpass(1, 500, 4000);
        popFrequency = 1.5;
        pop = ot.noises(4, x+2) * en.ar(attackDecay, attackDecay, ot.velvet_noise(1, popFrequency)) : popFilter : *(15);
        crackleFrequency = 10*warble;
        crackle = ot.noises(4, x+2) * en.ar(attackDecay*0.25, attackDecay*0.25, ot.velvet_noise(1, crackleFrequency)) : popFilter : *(10);
    };
};


// The filter should come before downsampling, i think, but it sounds so much better after.
addDownsampling = ba.downSample(globalSamplerate) : hardware_filter with {
    hardware_filter = fi.lowpass(8, min(MAX_FREQ, ot.NYQUIST_AT(globalSamplerate)));
};

addBitcrusher = _
	: ef.dryWetMixer(globalSamplerate < 44099.5, addDownsampling)
	: ef.dryWetMixer(globalBitdepth < 15.5, ba.bitcrusher(globalBitdepth));


// ==== REVERB ====
revTime = 0.05;//0.2; // 0.1 - 60 
revDamp = globalReverbDamp; //0 - 1
revSize = globalReverbSize;//1; //0.5 - 5
revDiffusion = globalReverbDiffusion;//0.5;
revFeedback = 0.4;
revModDepth = 0;//0.1;
revModFreq = 2;
rev = re.greyhole(revTime, revDamp, revSize, revDiffusion, revFeedback, revModDepth, revModFreq) : stereo(fi.highpass(1, globalReverbCutoff));
addReverb(fx) = +(fx<:rev:>_:*(globalReverbAmount));
addReverbFromSend = (si.bus(2), (_ <: rev : stereo(*(globalReverbAmount)) )) :> si.bus(2);

//addGlobalSaturation = ef.dryWetMixer(globalSaturation, dynamicHarshSaturation);
addGlobalSaturation = customSaturation(globalSaturation);

addGlobalFilters = fi.highpass(2, globalHighpass) : fi.lowpass(4, globalLowpass);



interleaveDualStereo = route(4,4,(1,1),(3,2),(2,3),(4,4));
globalNoise = (stereoVinylNoise, stereoTapeNoise) : interleaveDualStereo :> par(i, 2, it.interpolate_linear(globalNoiseType)) : stereo(*(globalNoiseAmount));
//globalNoise = stereoTapeNoise;
addGlobalNoise(l, r) = (l, r) <: (si.bus(2), (globalNoise : stereo(*(noiseLevel)))) : interleaveDualStereo :> stereo(+) with {
	noiseGateLevel = ba.db2linear(-80);
	noiseFloor = ba.db2linear(-24);
	noiseLevel = (l+r)/2 : an.amp_follower_ud(0.1/1000, 50/1000) : /(noiseGateLevel) : min(1) : *(noiseFloor); //track in mono
};

muuute = hslider("HACK", 0, 0, 1, 0.01);
// Output
//process(left, right, fx) = (left, right) : stereo(addReverb(fx) : addGlobalFilters : addBitcrusher : addGlobalSaturation : *(globalVolume));
process(left, right, fx) = (left, right, fx) : addReverbFromSend : addGlobalNoise : stereo(addGlobalFilters : addBitcrusher : addGlobalSaturation : *(globalVolume)) : ot.limiter_stereo;

